//////////////////////////////////////////////////////
/// The following example illustrates an Object Spaces example 
/// application developed using C# and the .NET Framework.
/// 
/// author: Gopalan Suresh Raj
/// Copyrights (c), 2002. All Rights Reserved.
/// URL: http://gsraj.tripod.com/
/// email: gopalan@gmx.net
/// 
/// Here's how to build the assemply
/// <compile>
/// C:\WINNT\Microsoft.NET\Framework\v1.0.3705\csc Teller.cs /r:system.dll /r:Microsoft.ObjectSpaces.dll /r:Account.dll 
/// </compile>
/// 
//////////////////////////////////////////////////////

using System;
using Microsoft.ObjectSpaces;

namespace Bank {
  /// <summary>
  /// Summary description for Teller.
  /// </summary>
  public class Teller {

    IObjectSpace objectSpaces_;

    public Teller(String sourceFile) {
      try {
        // Create an instance of an interface to the ObjectSpace.
        this.objectSpaces_ = ObjectSpaceFactory.CreateObjectSpace(sourceFile);
      }
      catch (Exception exception) {  
        // Handle any exceptions.
        Console.WriteLine(exception);
      }
    }

    /**
     * Create Checking or Savings Account
     */
    public void createAccount(string accountNo,
      String customerNames, 
      int startingBalance, 
      int accountType) {
      Account account = null;

      ////////////////////////////////////
      // Perform  ObjectSpaces operations
      ////////////////////////////////////
      try {
        account = (Account) this.objectSpaces_.CreateObject(typeof(Account), 
          accountNo, customerNames, accountType, startingBalance);
        // Commit the changes
        this.objectSpaces_.BeginTransaction();
        this.objectSpaces_.Update(account);
        this.objectSpaces_.CommitTransaction();
        Console.WriteLine(account.ToString()+"\nSuccessfully created.");
      } catch(Exception exception) {
        Console.WriteLine(exception);
        if(account != null) {
          this.objectSpaces_.RollbackTransaction();
        }
      }
    }
	
    /**
     * Delete and close the account
     */
    public void deleteAccount(string accountNumber) {

      Account account = null;
      string filter = "AccountNo='"+accountNumber+"'";

      ////////////////////////////////////
      // Perform  ObjectSpaces operations
      ////////////////////////////////////
      try {
        account = (Account) this.objectSpaces_.GetObject(typeof(Account), filter);
        if(account != null) {
          // Invoke Delete Persistent on the object
          this.objectSpaces_.DeleteObject(account);
        }
        // Commit the changes
        this.objectSpaces_.BeginTransaction();
        this.objectSpaces_.Update(account);
        this.objectSpaces_.CommitTransaction();
        Console.WriteLine("Successfully deleted Account No: "+accountNumber);
      } catch(Exception exception) {
        Console.WriteLine(exception);
        if(account != null) {
          this.objectSpaces_.RollbackTransaction();
        }
      }
    }

    /**
     * Transfer Money between Accounts
     */ 
    public void transferMoney(string fromAccountNo, string toAccountNo, int amount) {
      Account fromAccount = null;
      Account toAccount = null;
      string fromFilter = "AccountNo='"+fromAccountNo+"'";
      string toFilter = "AccountNo='"+toAccountNo+"'";

      ////////////////////////////////////
      // Perform  ObjectSpaces operations
      ////////////////////////////////////
      fromAccount = (Account) this.objectSpaces_.GetObject(typeof(Account), fromFilter);
      toAccount = (Account) this.objectSpaces_.GetObject(typeof(Account), toFilter);

      try {
        if((fromAccount != null) && (toAccount != null) ) {
          if(amount < fromAccount.Balance) {
            this.objectSpaces_.BeginTransaction();
            fromAccount.debit(amount);
            toAccount.credit(amount);
            // Commit the changes
            this.objectSpaces_.UpdateAll();
            this.objectSpaces_.CommitTransaction();
            Console.WriteLine("Successfully transfered $"+amount+
              " from Account No: "+fromAccountNo+
              " to Account No: "+toAccountNo);
          }
        }
      }
      catch(Exception exception) {
        Console.WriteLine(exception);
        if((fromAccount != null) || (toAccount != null)) {
          this.objectSpaces_.RollbackTransaction();
        }
      }
    }

    /**
     * List Account Details
     */
    public void listAccounts(string filter) {
                           
      ////////////////////////////////////
      // Perform  ObjectSpaces operations
      ////////////////////////////////////
      // Iterate through the list of Accounts.
      foreach(Account accounts in this.objectSpaces_.GetObjects(typeof(Account), filter)) {
        Console.WriteLine(accounts.ToString());
      }
    }

  }

  /**
   * Main Entry Point to the Object Space Application
   */ 
  public class Bank {
    public static void Main(string[] args) {

      // DbSource.xml is the default source file.
      string sourceFile = "Source.xml";

      // If an argument is supplied (for example, Bank.exe 'MyFile.xml'), use that argument
      // as the source file.
      if (args.Length > 0) {
        sourceFile = args[0];
      }
      Teller teller = new Teller(sourceFile);
      // Create Checking Accounts
      teller.createAccount("C1000", "Gopalan Suresh Raj", 1000000, Account.CHECKING_ACCOUNT_);
      teller.createAccount("C1001", "Athul Suresh Raj", 1000000, Account.CHECKING_ACCOUNT_);

      // Create Savings Accounts
      teller.createAccount("S1000", "Gopalan Suresh Raj", 1000000, Account.SAVINGS_ACCOUNT_);
      teller.createAccount("S1001", "Athul Suresh Raj", 1000000, Account.SAVINGS_ACCOUNT_);

      // List Checking Account Details
      string filter = "AccountType="+Account.CHECKING_ACCOUNT_;
      Console.WriteLine("\nListing All Checking Accounts\n-----------------------------");
      teller.listAccounts(filter);
      Console.WriteLine("-----------------------------");

      // List Savings Account Details
      filter = "AccountType="+Account.SAVINGS_ACCOUNT_;
      Console.WriteLine("\nListing All Savings Accounts\n----------------------------");
      teller.listAccounts(filter);
      Console.WriteLine("----------------------------");

      // Delete some Accounts
      teller.deleteAccount("C1000");
      teller.deleteAccount("S1001");

      // Perform Transfer Money operations on some accounts
      teller.transferMoney("S1000", "C1001", 10000);

      // List all Account Details
      filter = "";
      Console.WriteLine("\nListing All Accounts\n--------------------");
      teller.listAccounts(filter);
      Console.WriteLine("--------------------");
    
    }
  }
}
